<?php defined('_JEXEC') or die();
/**
 * @version		1.1.0
 * @package		rsiFAQ
 * @subpackage	Component
 * @copyright	Copyright (C) 2010 Rockford Solutions, Inc. All rights reserved.
 * @license		http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

jimport('joomla.application.component.model');
jimport('joomla.html.pagination');

abstract class rsiFAQModel extends JModel {
	protected $_resource = null;
	protected $_singular = null;
	protected $_parentid = null;
	protected $_rows     = null;
	protected $_count    = null;
	protected $_pages    = null;
	
	public function __construct($resource, $singular, $parentid = null) {
		parent::__construct();
		
		global $mainframe, $option;
		
		$this->_resource = $resource;
		$this->_singular = $singular;
		$this->_parentid = $parentid;
		
		$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
		$start = JRequest::getVar('limitstart', 0, null, 'int');
		$start = $limit != 0 ? floor($start / $limit) * $limit : 0;
		
		$this->setState('limit', $limit);
		$this->setState('start', $start);
		
		$filter_search = $mainframe->getUserStateFromRequest($option . '.' . $this->_resource . '_list.filter_search', 'filter_search',    '');
		$filter_status = $mainframe->getUserStateFromRequest($option . '.' . $this->_resource . '_list.filter_status', 'filter_status',    '');
		$filter_order  = $mainframe->getUserStateFromRequest($option . '.' . $this->_resource . '_list.filter_order',  'filter_order',     'order');
		$filter_dir    = $mainframe->getUserStateFromRequest($option . '.' . $this->_resource . '_list.filter_dir',    'filter_order_Dir', 'asc');
		
		$this->setState('filter_search', $filter_search);
		$this->setState('filter_status', $filter_status);
		$this->setState('filter_order',  $filter_order);
		$this->setState('filter_dir',    $filter_dir);
	}
	
	public function getRow() {
		$id = array_shift(JRequest::getVar('cid', array(), null, 'array'));
		
		$row =& $this->getTable();
		$row->load($id);
		
		return $row;
	}
	
	public function getRows() {
		if (is_null($this->_rows)) {
			$query = $this->_buildQuery();
			$start = $this->getState('start');
			$limit = $this->getState('limit');
			
			$this->_rows =& $this->_getList($query, $start, $limit);
		}
		
		return $this->_rows;
	}
	
	public function getCount() {
		if (is_null($this->_count)) {
			$query = $this->_buildQuery();
			
			$this->_count = $this->_getListCount($query);
		}
		
		return $this->_count;
	}
	
	public function getPages() {
		if (is_null($this->_pages)) {
			$count = $this->getCount();
			$start = $this->getState('start');
			$limit = $this->getState('limit');
			
			$this->_pages = new JPagination($count, $start, $limit);
		}
		
		return $this->_pages;
	}
	
	public function getOrder() {
		$id = array_shift(JRequest::getVar('cid', array(), null, 'array'));
		
		$table =& $this->getTable();
			   
		$query = 'SELECT `template`.`order`, `template`.`title` '
		       . (is_null($this->_parentid)
		       ? 'FROM `' . $table->getTableName() . '` AS `template` '
		       : 'FROM `' . $table->getTableName() . '` AS `instance` '
			   . 'INNER JOIN `' . $table->getTableName() . '` AS `template` '
			   . 'ON `instance`.`' . $this->_parentid . '` = `template`.`' . $this->_parentid . '` '
			   . 'WHERE `instance`.`' . $table->getKeyName() . '` = ' . (int) $id . ' ')
			   . 'ORDER BY `template`.`order` ASC';
		
		$rows =& $this->_getList($query);
		
		return $rows;
	}
	
	public function getSelected() {
		$cid = JRequest::getVar('cid', array(), null, 'array');
		
		$table =& $this->getTable();
		
		$query = 'SELECT `id`, `title` '
		       . 'FROM `' . $table->getTableName() . '` '
			   . 'WHERE `id` IN (' . implode(', ', $cid) . ')';
		
		return $this->_getList($query);
	}
	
	protected abstract function _buildQuery();
	
	public function store() {
		$data = JRequest::getVar($this->_singular, array(), null, 'array');
		$data['updated'] = date('Y-m-d H:i:s');
		$data['created'] = empty($data['id']) ? $data['updated'] : null;
		
		$row =& $this->getTable();
		
		if (!$row->bind($data)) return JError::raiseWarning(500, $row->getError());
		if (!$row->check()) return JError::raiseWarning(500, $row->getError());
		if (!$row->store()) return JError::raiseWarning(500, $row->getError());
		
		return $row;
	}
	
	public function delete() {
		$cid = JRequest::getVar('cid', array(), null, 'array');
		
		$row =& $this->getTable();
		
		foreach ($cid as $id) {
			if (!$row->delete($id)) return JError::raiseWarning(500, $row->getError());
		}
		
		return true;
	}
	
	public function saveorder() {
		$cid   = JRequest::getVar('cid',   array(), null, 'array');
		$order = JRequest::getVar('order', array(), null, 'array');
		$order = array_combine($cid, $order);
		
		if (is_null($this->_parentid)) {
			asort($order);
			
			foreach (array_keys($order) as $index => $id) {
				if (!$this->_setOrder($id, $index + 1)) return false;
			}
		} else {
			$table =& $this->getTable();
			
			$query = 'SELECT `id`, `' . $this->_parentid . '` AS `parent_id` '
				   . 'FROM `' . $table->getTableName() . '` ';
			
			$rows =& $this->_getList($query);
			
			$parent_order = array();
			
			foreach ($rows as $row) {
				if (isset($order[$row->id])) {
					if (!isset($parent_order[$row->parent_id])) {
						$parent_order[$row->parent_id] = array();
					}
					
					$parent_order[$row->parent_id][$row->id] = $order[$row->id];
				}
			}
			
			foreach ($parent_order as $parent_id => $order) {
				asort($order);
			
				foreach (array_keys($order) as $index => $id) {
					if (!$this->_setOrder($id, $index + 1)) return false;
				}
			}
		}
		
		return true;
	}
	
	public function orderup() {
		$id = array_shift(JRequest::getVar('cid', array(), null, 'array'));
		
		$table =& $this->getTable();
		
		$query = 'SELECT `template`.`id`, `template`.`order` '
			   . 'FROM `' . $table->getTableName() . '` AS `instance` '
			   . 'INNER JOIN `' . $table->getTableName() . '` AS `template` '
			   . 'ON `instance`.`order` >= `template`.`order` '
			   . (is_null($this->_parentid) ? ''
			   : 'AND `instance`.`' . $this->_parentid . '` = `template`.`' . $this->_parentid . '` ')
		       . 'WHERE `instance`.`' . $table->getKeyName() . '` = ' . (int) $id . ' '
			   . 'ORDER BY `template`.`order` DESC '
			   . 'LIMIT 2';
		
		$rows =& $this->_getList($query);
		
		if (!$this->_setOrder($rows[0]->id, $rows[1]->order)) return false;
		if (!$this->_setOrder($rows[1]->id, $rows[0]->order)) return false;
		
		return true;
	}
	
	public function orderdown() {
		$id = array_shift(JRequest::getVar('cid', array(), null, 'array'));
		
		$table =& $this->getTable();
		
		$query = 'SELECT `template`.`id`, `template`.`order` '
			   . 'FROM `' . $table->getTableName() . '` AS `instance` '
			   . 'INNER JOIN `' . $table->getTableName() . '` AS `template` '
			   . 'ON `instance`.`order` <= `template`.`order` '
			   . (is_null($this->_parentid) ? ''
			   : 'AND `instance`.`' . $this->_parentid . '` = `template`.`' . $this->_parentid . '` ')
		       . 'WHERE `instance`.`' . $table->getKeyName() . '` = ' . (int) $id . ' '
			   . 'ORDER BY `template`.`order` ASC '
			   . 'LIMIT 2';
		
		$rows =& $this->_getList($query);
		
		if (!$this->_setOrder($rows[0]->id, $rows[1]->order)) return false;
		if (!$this->_setOrder($rows[1]->id, $rows[0]->order)) return false;
		
		return true;
	}
	
	public function move() {
		if (is_null($this->_parentid)) return false;
		
		$cid = JRequest::getVar('cid', array(), null, 'array');
		$pid = JRequest::getVar('pid', array(), null, 'array');
		
		$data = array('updated' => date('Y-m-d H:i:s'), $this->_parentid => (int) array_shift($pid));

		$row =& $this->getTable();
		
		foreach ($cid as $id) {
			$data['id'] = (int) $id;
			
			if (!$row->bind($data)) return JError::raiseWarning(500, $row->getError());
			if (!$row->check()) return JError::raiseWarning(500, $row->getError());
			if (!$row->store()) return JError::raiseWarning(500, $row->getError());
		}
		
		return true;
	}
	
	public function publish() {
		return $this->_setStatus(1);
	}
	
	public function unpublish() {
		return $this->_setStatus(0);
	}
	
	public function accesspublic() {
		return $this->_setAccess(0);
	}
	
	public function accessregistered() {
		return $this->_setAccess(1);
	}
	
	public function accessspecial() {
		return $this->_setAccess(2);
	}
	
	protected function _setStatus($status) {
		$cid = JRequest::getVar('cid', array(), null, 'array');
		
		$data = array('updated' => date('Y-m-d H:i:s'), 'status' => (int) $status);
		
		$row =& $this->getTable();
		
		foreach ($cid as $id) {
			$data['id'] = (int) $id;
			
			if (!$row->bind($data)) return JError::raiseWarning(500, $row->getError());
			if (!$row->check()) return JError::raiseWarning(500, $row->getError());
			if (!$row->store()) return JError::raiseWarning(500, $row->getError());
		}
		
		return true;
	}
	
	protected function _setAccess($access) {
		$cid = JRequest::getVar('cid', array(), null, 'array');
		
		$data = array('updated' => date('Y-m-d H:i:s'), 'access' => (int) $access);
		
		$row =& $this->getTable();
		
		foreach ($cid as $id) {
			$data['id'] = (int) $id;
			
			if (!$row->bind($data)) return JError::raiseWarning(500, $row->getError());
			if (!$row->check()) return JError::raiseWarning(500, $row->getError());
			if (!$row->store()) return JError::raiseWarning(500, $row->getError());
		}
		
		return true;
	}
	
	protected function _setOrder($id, $order) {
		$data = array('id' => (int) $id, 'updated' => date('Y-m-d H:i:s'), 'order' => (int) $order);
		
		$row  =& $this->getTable();
		
		if (!$row->bind($data)) return JError::raiseWarning(500, $row->getError());
		if (!$row->check()) return JError::raiseWarning(500, $row->getError());
		if (!$row->store()) return JError::raiseWarning(500, $row->getError());
		
		return true;
	}
	
	protected function &_getObject($query) {
		$this->_db->setQuery($query);
		$result = $this->_db->loadObject();
		
		return $result;
	}
}
