<?php defined('_JEXEC') or die();
/**
 * @version		1.1.0
 * @package		rsiFAQ
 * @subpackage	Component
 * @copyright	Copyright (C) 2010 Rockford Solutions, Inc. All rights reserved.
 * @license		http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

class rsiFAQModelCategories extends rsiFAQModel {
	public function __construct() {
		parent::__construct('categories', 'category', 'section_id');
		
		global $mainframe, $option;
		
		$filter_section_id = $mainframe->getUserStateFromRequest($option . '.categories_list.filter_section_id', 'filter_section_id', '');
		
		$this->setState('filter_section_id', $filter_section_id);
	}
	
	protected function _buildQuery() {
		$query = 'SELECT `categories`.*, '
		       . '`sections`.`title` AS `section_title`, '
		       . 'COUNT(`questions`.`id`) AS `questions` '
		       . 'FROM `#__rsi_faq_sections` AS `sections` '
			   . 'INNER JOIN `#__rsi_faq_categories` AS `categories` '
			   . 'ON `sections`.`id` = `categories`.`section_id` '
			   . 'LEFT JOIN `#__rsi_faq_questions` AS `questions` '
			   . 'ON `categories`.`id` = `questions`.`category_id` '
			   . $this->_buildQueryWhere() . ' '
			   . 'GROUP BY `categories`.`id` '
			   . $this->_buildQueryOrderBy();
			   //$this->_db->setQuery($query); echo '<div>' . $this->_db->getQuery() . '</div>';
		
		return $query;
	}
	
	protected function _buildQueryWhere() {
		$search = $this->getState('filter_search');
		$status = $this->getState('filter_status');
		
		$section_id = $this->getState('filter_section_id');
		
		$where = array();
		
		if ($search !== '') {
			$keywords = explode(' ', trim($search));
			
			$ids    = array();
			$titles = array();
			
			foreach ($keywords as $keyword) {
				if ($keyword !== '') {
					$ids[]    = '`categories`.`id` LIKE \'%' . $keyword . '%\'';
					$titles[] = '`categories`.`title` LIKE \'%' . $keyword . '%\'';
				}
			}
			
			$where[] = '(' . implode(' OR ', $ids) . ' OR ' . implode(' OR ', $titles) . ')';
		}
		
		if ($status !== '') {
			$where[] = '`categories`.`status` = ' . (int) $status;
		}
		
		if ($section_id !== '') {
			$where[] = '`categories`.`section_id` = ' . (int) $section_id;
		}
		
		return empty($where) ? '' : 'WHERE ' . implode(' AND ', $where);
	}
	
	protected function _buildQueryOrderBy() {
		$order = $this->getState('filter_order');
		$dir   = $this->getState('filter_dir');
		
		switch ($order) {
			case 'order':
				$order_by = '`sections`.`order` ' . $dir . ', `categories`.`order` ' . $dir;
			break;
			case 'section_title':
				$order_by = '`sections`.`title` ' . $dir . ', `categories`.`title` ' . $dir;
			break;
			default:
				$order_by = '`categories`.`' . $order . '` ' . $dir;
			break;
		}
		
		return 'ORDER BY ' . $order_by;
	}
	
	public function getSections() {
		$query = 'SELECT `id`, `title` '
		       . 'FROM `#__rsi_faq_sections` '
			   . 'ORDER BY `order`';
			   
		return $this->_getList($query);
	}
}
