<?php defined('_JEXEC') or die();
/**
 * @version		1.1.0
 * @package		rsiFAQ
 * @subpackage	Component
 * @copyright	Copyright (C) 2010 Rockford Solutions, Inc. All rights reserved.
 * @license		http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

class rsiFAQModelQuestions extends rsiFAQModel {
	public function __construct() {
		parent::__construct('questions', 'question', 'category_id');
		
		global $mainframe, $option;
		
		$filter_section_id  = $mainframe->getUserStateFromRequest($option . '.events_list.filter_section_id',  'filter_section_id',  '');
		$filter_category_id = $mainframe->getUserStateFromRequest($option . '.events_list.filter_category_id', 'filter_category_id', '');
		
		$this->setState('filter_section_id',  $filter_section_id);
		$this->setState('filter_category_id', $filter_category_id);
	}
	
	protected function _buildQuery() {
		$query = 'SELECT `questions`.*, `categories`.`section_id`, '
		       . '`sections`.`title` AS `section_title`, '
		       . '`categories`.`title` AS `category_title` '
		       . 'FROM `#__rsi_faq_sections` AS `sections` '
			   . 'INNER JOIN `#__rsi_faq_categories` AS `categories` '
			   . 'ON `sections`.`id` = `categories`.`section_id` '
			   . 'INNER JOIN `#__rsi_faq_questions` AS `questions` '
			   . 'ON `categories`.`id` = `questions`.`category_id` '
			   . $this->_buildQueryWhere() . ' '
			   . $this->_buildQueryOrderBy();
		
		return $query;
	}
	
	protected function _buildQueryWhere() {
		$search = $this->getState('filter_search');
		$status = $this->getState('filter_status');
		
		$section_id  = $this->getState('filter_section_id');
		$category_id = $this->getState('filter_category_id');
		
		$where = array();
		
		if ($search !== '') {
			$keywords = explode(' ', trim($search));
			
			$ids    = array();
			$titles = array();
			
			foreach ($keywords as $keyword) {
				if ($keyword !== '') {
					$ids[]    = '`questions`.`id` LIKE \'%' . $keyword . '%\'';
					$titles[] = '`questions`.`title` LIKE \'%' . $keyword . '%\'';
				}
			}
			
			$where[] = '(' . implode(' OR ', $ids) . ' OR ' . implode(' OR ', $titles) . ')';
		}
		
		if ($status !== '') {
			$where[] = '`questions`.`status` = ' . (int) $status;
		}
		
		if ($category_id !== '') {
			$where[] = '`questions`.`category_id` = ' . (int) $category_id;
		} elseif ($section_id !== '') {
			$where[] = '`categories`.`section_id` = ' . (int) $section_id;
		}
		
		return empty($where) ? '' : 'WHERE ' . implode(' AND ', $where);
	}
	
	protected function _buildQueryOrderBy() {
		$order = $this->getState('filter_order');
		$dir   = $this->getState('filter_dir');
		
		switch ($order) {
			case 'section_title':
				$order_by = '`sections`.`title` ' . $dir . ', `categories`.`title` ' . $dir;
			break;
			case 'category_title':
				$order_by = '`categories`.`title` ' . $dir;
			break;
			default:
				$order_by = '`questions`.`' . $order . '` ' . $dir;
			break;
		}
		
		return 'ORDER BY ' . $order_by;
	}
	
	public function getOrder() {
		$id = array_shift(JRequest::getVar('cid', array(), null, 'array'));
		
		$query = 'SELECT `template`.`order`, `template`.`question` '
		       . 'FROM `#__rsi_faq_questions` AS `instance` '
			   . 'INNER JOIN `#__rsi_faq_questions` AS `template` '
			   . 'ON `instance`.`category_id` = `template`.`category_id` '
			   . 'WHERE `instance`.`id` = ' . (int) $id . ' '
			   . 'ORDER BY `template`.`order` ASC';
		
		$rows =& $this->_getList($query);
		echo $this->_db->getErrorMsg();
		
		return $rows;
	}
	
	public function getSelected() {
		$cid = JRequest::getVar('cid', array(), null, 'array');
		
		$table =& $this->getTable();
		
		$query = 'SELECT `id`, `question` '
		       . 'FROM `' . $table->getTableName() . '` '
			   . 'WHERE `id` IN (' . implode(', ', $cid) . ')';
		
		return $this->_getList($query);
	}
	
	public function getSections() {
		$query = 'SELECT `id`, `title` '
		       . 'FROM `#__rsi_faq_sections` '
			   . 'ORDER BY `order`';
			   
		return $this->_getList($query);
	}
	
	public function getCategories() {
		$section_id = $this->getState('filter_section_id');
		
		$query = 'SELECT `categories`.`id`, `categories`.`title`, '
		       . '`categories`.`section_id`, `sections`.`title` AS `section_title` '
		       . 'FROM `#__rsi_faq_sections` AS `sections` '
			   . 'INNER JOIN `#__rsi_faq_categories` AS `categories` '
			   . 'ON `sections`.`id` = `categories`.`section_id` ';
			   
		if ($section_id !== '') $query .= 'WHERE `sections`.`id` = ' . (int) $section_id . ' ';
		
		$query .= 'ORDER BY `sections`.`order`, `categories`.`order`';
			   
		return $this->_getList($query);
	}
}
