<?php
/**
* VirtueMart Categories Module
* NOTE: THIS MODULE REQUIRES THE PHPSHOP COMPONENT!
*
* @copyright (C) 2010 JFactory Project
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
* This extention is Free Software.
*/

// no direct access
defined('_JEXEC') or die('Restricted access');

require_once (JPATH_SITE.DS.'modules'.DS.'mod_virtuemart_categories'.DS.'classes'.DS.'tree.php');

class VirtuemartCategories extends JObject
{
	/**
	 * menu items stored as tree
	 *
	 * @access private
	 * @param array
	 */
	var $_tree = null;

	/**
	 * active item
	 *
	 * @access private
	 * @param array
	 */
	var $_active = 0;

	/**
	 * class constructor
	 *
	 * @access public
	 */
	function __construct()
	{
		$this->_buildTree();
		$this->_active = JRequest::getInt('category_id', 0);
	}

	/**
	 * method to get categories from table
	 *
	 * @access private
	 * @return array
	 */
	function _load()
	{
		$database =& JFactory::getDBO();

		$query = 'SELECT c.category_id as id, x.category_parent_id as parent, c.category_name as name, c.category_thumb_image as image '
						.'FROM #__vm_category AS c '
						.'LEFT JOIN #__vm_category_xref AS x '
						.'ON c.category_id = x.category_child_id '
						.'WHERE c.category_publish = "Y" '
						.'ORDER BY x.category_parent_id, c.list_order '
						;

		$database->setQuery($query);

		return $database->loadObjectList();
	}

	/**
	 * method to build menu tree
	 *
	 * @access private
	 */
	function _buildTree($maxdepth = 10)
	{
		$cache = &JFactory::getCache('mod_virtuemart_categories', 'output');

		if (!$data = $cache->get('tree')) {

			$this->_tree = new VMTree();
			$rows = $this->_load();

			$ids = array();
			$ids[0] = true;
			$last = null;
			$unresolved = array();
			// pop the first item until the array is empty if there is any item
			if ( is_array($rows)) {
				while (count($rows) && !is_null($row = array_shift($rows)))
				{
					if (array_key_exists($row->parent, $ids)) {
						$this->_tree->addNode($row);
						// record loaded parents
						$ids[$row->id] = true;
					} else {
						// no parent yet so push item to back of list
						// SAM: But if the key isn't in the list and we dont _add_ this is infinite, so check the unresolved queue
						if(!array_key_exists($row->id, $unresolved) || $unresolved[$row->id] < $maxdepth) {
							array_push($rows, $row);
							// so let us do max $maxdepth passes
							// TODO: Put a time check in this loop in case we get too close to the PHP timeout
							if(!isset($unresolved[$row->id])) $unresolved[$row->id] = 1;
							else $unresolved[$row->id]++;
						}
					}
				}
			}
			$cache->store(serialize($this->_tree), 'tree');
		} else {
			$this->_tree = unserialize($data);
		}
	}

	/**
	 * Returns a reference to a VMCategories object
	 *
	 * @access	public
	 * @return VMCategories 	A menu object.
	 */
	function &getInstance()
	{
		static $instance;

		if (!isset( $instance )) {
			$instance = new VirtuemartCategories();
		}

		return $instance;
	}

	/**
	 * Get menu tree object
	 *
	 * @access public
	 * @return mixed The item object, or null
	 */
	function &getTree()
	{
		return $this->_tree;
	}

	/**
	 * Get active menu item from tree
	 *
	 * @access public
	 * @return mixed The tree node, or null if not found
	 */
	function &getActive()
	{
		return $this->_tree->getNode($this->_active);
	}

}